@@ -23,9 +23,7 @@
 #'   \code{coef_test()}, \code{"saddlepoint"} is not currently supported in
 #'   \code{conf_int()} because saddlepoint confidence intervals do not have a
 #'   closed-form solution.
-#' @param p_values Logical indicating whether to report p-values. The default
-#'   value is \code{FALSE}.
-
+#' 
 #' @inheritParams coef_test
 #'
 #' @return A data frame containing estimated regression coefficients, standard
@@ -74,7 +72,7 @@ conf_int <- function(obj, vcov, level = .95, test = "Satterthwaite", coefs = "Al
                z = Inf,
                `naive-t` = nlevels(attr(vcov, "cluster")) - 1,
                `naive-tp` = nlevels(attr(vcov, "cluster")) - p,
-               `Satterthwaite` = Satterthwaite(beta = beta, SE = SE, P_array = P_array)$df
+               `Satterthwaite` = Satterthwaite_df(P_array = P_array)
   )
 
   crit <- qt(1 - (1 - level) / 2, df = df)
@@ -97,6 +95,7 @@ conf_int <- function(obj, vcov, level = .95, test = "Satterthwaite", coefs = "Al
   class(result) <- c("conf_int_clubSandwich", class(result))
   attr(result, "type") <- attr(vcov, "type")
   attr(result, "level") <- level
+  
   result
 }
 
@@ -129,6 +128,10 @@ conf_int <- function(obj, vcov, level = .95, test = "Satterthwaite", coefs = "Al
 #'   closed-form solution.
 #' @param p_values Logical indicating whether to report p-values. The default
 #'   value is \code{FALSE}.
+#' @param adjustment_method Correction method, a \code{\link{character}} string
+#'   from \code{p.adjust.methods}, which is passed to \code{\link{p.adjust}} to
+#'   correct p-values in the case of multiple comparisons. Defaults to
+#'   \code{"none"}. Any value given will be ignored if \code{p_values = FALSE}.
 #' @inheritParams coef_test
 #' 
 #' @details Constraints can be specified directly as q X p matrices or
@@ -164,6 +167,11 @@ conf_int <- function(obj, vcov, level = .95, test = "Satterthwaite", coefs = "Al
 #'                   contrasts = constrain_pairwise(":education", reg_ex = TRUE),
 #'                   test = "Satterthwaite")
 #'  
+#'   # Repeat the above, but with p-values displayed and multiple comparisons adjustment applied.
+#'   linear_contrast(Duncan_fit, vcov = "CR2", cluster = Duncan$cluster,
+#'                   contrasts = constrain_pairwise(":education", reg_ex = TRUE),
+#'                   test = "Satterthwaite", p_values = TRUE, adjustment_method = "hochberg")
+#'  
 #'   # Pairwise comparisons of type-by-income interactions
 #'   linear_contrast(Duncan_fit, vcov = "CR2", cluster = Duncan$cluster,
 #'                   contrasts = constrain_pairwise(":income", reg_ex = TRUE, with_zero = TRUE),
@@ -173,7 +181,7 @@ conf_int <- function(obj, vcov, level = .95, test = "Satterthwaite", coefs = "Al
 #'
 #' @export
 
-linear_contrast <- function(obj, vcov, contrasts, level = .95, test = "Satterthwaite", ..., p_values = FALSE) {
+linear_contrast <- function(obj, vcov, contrasts, level = .95, test = "Satterthwaite", ..., p_values = FALSE, adjustment_method = "none") {
   
   if (level <= 0 | level >= 1) stop("Confidence level must be between 0 and 1.")
   
@@ -254,7 +262,11 @@ linear_contrast <- function(obj, vcov, contrasts, level = .95, test = "Satterthw
       GH$H <- array_multiply(contrasts, GH$H)
     } else {
       H <- array(NA, dim = c(3, q, dims[3:4]))
-      for (i in 1:dims[1]) H[i,,,] <- array_multiply(contrasts, GH$H[i,,,,drop=FALSE])
+      for (i in 1:dims[1]) {
+        Hi <- GH$H[i,,,]
+        dim(Hi) <- dims[-1L]
+        H[i,,,] <- array_multiply(contrasts, Hi)
+      }
       GH$H <- H
     }
     P_array <- get_P_array(GH = GH)
@@ -264,7 +276,7 @@ linear_contrast <- function(obj, vcov, contrasts, level = .95, test = "Satterthw
                z = Inf,
                `naive-t` = nlevels(attr(vcov, "cluster")) - 1,
                `naive-tp` = nlevels(attr(vcov, "cluster")) - p,
-               `Satterthwaite` = Satterthwaite(beta = est, SE = SE, P_array = P_array)$df
+               `Satterthwaite` = Satterthwaite_df(P_array = P_array)
   )
   
   crit <- qt(1 - (1 - level) / 2, df = df)
@@ -279,14 +291,28 @@ linear_contrast <- function(obj, vcov, contrasts, level = .95, test = "Satterthw
   )
   row.names(result) <- result$Coef
   
+  # p-value adjustment method
+  adjustment_method <- match.arg(adjustment_method, p.adjust.methods, several.ok = FALSE)
+  
   if (p_values) {
     t_stat <- result$Est / result$SE
     result$p_val <- 2 * pt(abs(t_stat), df = result$df, lower.tail = FALSE)
+    
+    # p-value adjustment
+    if (adjustment_method != "none" & nrow(result) == 1) {
+      warning("Only one p-value is available. No p-value adjustment will be performed.") # warning by copilot
+    } else if (adjustment_method != "none") {
+      result$p_val <- p.adjust(result$p_val, method = adjustment_method)
+    }
+    
+  } else if (adjustment_method != "none") {
+    warning("p_values = FALSE, so p-value adjustment is not applied.") # warning by copilot
   }
   
   class(result) <- c("conf_int_clubSandwich", class(result))
   attr(result, "type") <- attr(vcov, "type")
   attr(result, "level") <- level
+  
   result
 }
 
